/* Sender_demo_1
  zyklische Übertragung einiger Testdaten
*/

// Bibliotheken
#include <DataTransmitter.h>

// Daten, die übertragen werden sollen
const float Pi = 3.14159; // erstes Element, eine Konstante vom Typ Float
unsigned long system_time; // zweites Element, die aktuelle Systemlaufzeit in ms
char test_string[40] = "Amateurfunk ist ein vielseitiges Hobby."; // drittes Element, eine Zeichenkette

// einige Variablen & Konstanten
const byte TRANSMITTER_PIN = 12; // das (digitale) Pin, an welchem die Daten ausgegeben werden
const byte BYTE_RATE = 30; // die Übertragungsgeschwindigkeit (4...255 Bytes/s)
const byte LOOKUP_TABLE_START = 0; // EEPROM Startadresse zur Speicherung des 256 Byte CRC-lookup table
const byte LED = 13; // die onboard-LED

dset_t dataset[2]; // ein Array mit zwei Elementen von Typ dset_t, welche jeweils einen Datensatz charakterisieren

byte dset[8]; // ein Array von 8 Byte, in welchem die Elemente 1 und 2 ("Pi" und "system_time") gebündelt übertragen
              // werden sollen

//#################################################################################################################

void setup() {
  
/*
Initialisierung der CRC8-Bibliothek (wird durch DataTransmitter.h automatisch mit eingebunden)

Parameter:
 - EPROM Startadresse zur Speicherung des 256 Byte CRC-lookup table

 Rückgabe: die nächste freie EEPROM-Adresse
*/
  CRC.init(LOOKUP_TABLE_START);
  
/*
Initialisierung der DataTransmitter-Bibliothek

Parameter (v.l.n.r.):
 - Datenrate in Byte/s
 - das (digitale) Pin, an welchem die Daten ausgegeben werden
 - "IS_LOW_ACTIVE" -> "True" oder "1" bedeutet inverse Logik (Sender aktiv, wenn Ausgang auf Masse)
*/
  DT.init(BYTE_RATE, TRANSMITTER_PIN, 0);

// dem Element "dataset_0" werden die (frei gewählte) ID 10 und die Datenquelle "dset" zugeordnet
  dataset[0] = DT.createDataset(dset, 10, sizeof(dset));
 // dem Element "dataset_1" werden die (frei gewählte) ID 11 und die Datenquelle "test_string" zugeordnet                     
  dataset[1] = DT.createDataset(test_string, 11, sizeof(test_string));
  
// "dataset[0]" wird mit Inhalt geladen (das 4 Byte Float-Element "Pi" beginnend bei Byte 0)
  DT.dataTransfer(&Pi, sizeof(Pi), dataset[0], 0, 1);

}

//###################################################################################################

void loop() {

// erfasse die Systemzeit        
  system_time = millis();
// kopiere die Systemzeit nach "dataset[0]" gleich hinter "Pi", beginnend bei Byte 4
  DT.dataTransfer(&system_time, sizeof(system_time), dataset[0], 4, 1);

// Datensatz 0 mit "Pi" und "system_time" wird ausgesandt
  DT.transmitData(dataset[0]);
  
// 5s Pause (die onboard-LED leuchtet für 500ms, um eine Übertragung zu signalisieren)
  digitalWrite(LED, HIGH);
  delay(500);
  digitalWrite(LED, LOW);
  delay(4500);

// Datensatz 1 mit "test_string" wird ausgesandt  
  DT.transmitData(dataset[1]);

// 5s Pause (die onboard-LED leuchtet für 500ms, um eine Übertragung zu signalisieren)
  digitalWrite(LED, HIGH);
  delay(500);
  digitalWrite(LED, LOW);
  delay(4500);

}
